﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HomeFinance.Model
{
    public class Categoria
    {
        private readonly string _nome;
        private Categoria _parent;
        private readonly List<Categoria> _children;

        public Categoria(string nome)
        {
            if (String.IsNullOrEmpty(nome))
                throw new ArgumentException("String.IsNullOrEmpty(nome)");
            _nome = nome;
            _children = new List<Categoria>();
        }

        public string Nome
        {
            get { return _nome; }
        }

        public string NomeCompleto
        {
            get { return ((Parent == null) ? "" : Parent.NomeCompleto + "/") + Nome; }
        }

        public Categoria Parent
        {
            get { return _parent; }
            set
            {
                if (value != _parent)
                {
                    if (_parent != null)
                        _parent.Remove(this);
                    if (value != null)
                        value.Add(this);
                }
            }
        }

        public IEnumerable<Categoria> Children
        {
            get { return _children; }
        }

        public bool IsAscendantOf(Categoria categoria)
        {
            if (categoria == null)
                return false;
            if (categoria.Parent == this || IsAscendantOf(categoria.Parent))
                return true;
            return false;
        }

        public bool IsDescendantOf(Categoria categoria)
        {
            if (categoria == null)
                return false;
            foreach (Categoria child in categoria.Children)
            {
                if (child == this || IsDescendantOf(child))
                    return true;
            }
            return false;
        }

        public IEnumerable<Categoria> GetDescendants(bool withRoot)
        {
            List<Categoria> categorie = new List<Categoria>();
            if (withRoot)
                categorie.Add(this);
            foreach (Categoria child in Children)
            {
                categorie.AddRange(child.GetDescendants(true));
            }
            return categorie;
        }

        public void Add(Categoria child)
        {
            if (child == null)
                throw new ArgumentNullException("child");
            if (child.Parent != null)
                throw new ArgumentException("child.Parent != null");
            _children.Add(child);
            child._parent = this;
        }

        public void Remove(Categoria child)
        {
            if (child == null)
                throw new ArgumentNullException("child");
            if (child.Parent != this)
                throw new ArgumentException("child.Parent != this");
            if (!Children.Contains(child))
                throw new ArgumentException("!Children.Contains(child)");
            _children.Remove(child);
            child._parent = null;
        }

        public override string ToString()
        {
            return String.Format("Categoria {0}", NomeCompleto);
        }
    }
}
