﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using HomeFinance.Persistence;

namespace HomeFinance.Model
{
    partial class Documento
    {
        private Categoria _categoriaRoot;
        private List<Movimento> _movimenti;
        private List<Selezione> _selezioni;

        private static Documento _instance;

        public event EventHandler Changed;

        private Documento()
        {
            New();
        }

        public static Documento GetInstance()
        {
            if (_instance == null)
                _instance = new Documento();
            return _instance;
        }

        public Categoria CategoriaRoot
        {
            get { return _categoriaRoot; }
        }

        public IEnumerable<Movimento> Movimenti
        {
            get { return _movimenti; }
        }

        public void Add(Movimento movimento)
        {
            if (movimento == null)
                throw new ArgumentNullException("movimento");
            _movimenti.Add(movimento);
            Invalidate();
        }

        public IEnumerable<MovimentoReale> GetMovimentiReali()
        {
            return _movimenti.OfType<MovimentoReale>();
        }

        public ISelezione CreateSelezione()
        {
            Selezione selezione = new Selezione();
            _selezioni.Add(selezione);
            return selezione;
        }

        public void New()
        {
            _categoriaRoot = new Categoria("HomeFinance");
            _movimenti = new List<Movimento>();
            _selezioni = new List<Selezione>();
            OnChanged();
        }

        public void Load(IDocumentoPersister persister)
        {
            if (persister == null)
                throw new ArgumentNullException("persister");
            IDocumentoLoader loader = persister.GetLoader();
            _categoriaRoot = loader.LoadCategorie();
            _movimenti = loader.LoadMovimenti();
            _selezioni = new List<Selezione>();
            OnChanged();
        }

        public void Save(IDocumentoPersister persister)
        {
            if (persister == null)
                throw new ArgumentNullException("persister");
            persister.Save(this);
        }

        private void Invalidate()
        {
            foreach (Selezione selezione in _selezioni)
                selezione.Invalidate();
            OnChanged();
        }

        protected virtual void OnChanged()
        {
            if (Changed != null)
                Changed(this, EventArgs.Empty);
        }

        #region Supporto per la generazione di movimenti _random

        private readonly Random _random = new Random();

        public IEnumerable<Movimento> CreaMovimentiRealiRandom(int count)
        {
            List<Categoria> categorie = CategoriaRoot.GetDescendants(true).ToList();
            List<Movimento> movimenti = new List<Movimento>();
            for (int k = 1; k <= count; k++)
            {
                movimenti.Add(new MovimentoReale(categorie[_random.Next(categorie.Count)],
                    (TipoMovimento) _random.Next(1, 3), String.Empty, _random.Next(10, 301),
                    DateTime.Today - new TimeSpan(_random.Next(32), 0, 0, 0)));
            }
            return movimenti;
        }

        public void AggiungiMovimentoRealeRandom()
        {
            Add(CreaMovimentiRealiRandom(1).ElementAt(0));
        }

        #endregion
    }
}
