﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HomeFinance.Model
{
    public abstract class Movimento
    {
        private readonly Categoria _categoria;
        private readonly TipoMovimento _tipo;
        private readonly string _descrizione;
        private readonly double _importo;

        protected Movimento(Categoria categoria, TipoMovimento tipo, string descrizione, double importo)
        {
            if (categoria == null)
                throw new ArgumentNullException("categoria");
            if (tipo != TipoMovimento.Ordinario && tipo != TipoMovimento.Straordinario)
                throw new ArgumentException("tipo != TipoMovimento.Ordinario && tipo != TipoMovimento.Straordinario");
            if (importo <= 0.0)
                throw new ArgumentException("importo <= 0.0");
            _categoria = categoria;
            _tipo = tipo;
            _descrizione = descrizione ?? String.Empty;
            _importo = importo;
        }

        public Categoria Categoria
        {
            get { return _categoria; }
        }

        public TipoMovimento Tipo
        {
            get { return _tipo; }
        }

        public string Descrizione
        {
            get { return _descrizione; }
        }

        public double Importo
        {
            get { return _importo; }
        }
    }
}
