﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HomeFinance.Model
{
    static class ValutatoreImportoFactory
    {
        public static IValutatoreImporto CreatePerImportoCostante()
        {
            return ValutatoreImportoCostante.Instance;
        }

        public static IValutatoreImporto CreatePerImportoVariabilePercentuale(double percentuale)
        {
            return new ValutatoreImportoVariabilePercentuale(percentuale);
        }

        class ValutatoreImportoCostante : IValutatoreImporto
        {
            public static readonly ValutatoreImportoCostante Instance = new ValutatoreImportoCostante();

            private ValutatoreImportoCostante()
            {
            }

            public virtual double CalcolaImporto(double importo)
            {
                if (importo <= 0.0)
                    throw new ArgumentException("importo <= 0.0");
                return importo;
            }
        }

        abstract class ValutatoreImportoVariabile : IValutatoreImporto
        {
            private double _importoCorrente = 0.0;

            public virtual double CalcolaImporto(double importo)
            {
                if (importo <= 0.0)
                    throw new ArgumentException("importo <= 0.0");
                if (_importoCorrente == 0.0)
                {
                    _importoCorrente = importo;
                }
                else
                {
                    _importoCorrente = CalcolaNuovoImporto(_importoCorrente);
                }
                return _importoCorrente;
            }

            protected abstract double CalcolaNuovoImporto(double importo);
        }

        class ValutatoreImportoVariabilePercentuale : ValutatoreImportoVariabile
        {
            private readonly double _percentuale;

            public ValutatoreImportoVariabilePercentuale(double percentuale)
            {
                if (percentuale == 0.0)
                    throw new ArgumentException("percentuale == 0.0");
                _percentuale = percentuale;
            }

            protected override double CalcolaNuovoImporto(double importo)
            {
                return importo * (1 + _percentuale);
            }
        }
    }
}