﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HomeFinance.Model
{
    static class ValutatoreScadenzaFactory
    {
        public static IValutatoreScadenza CreatePerScadenzaFissa(DateTime dataScadenza)
        {
            return new ValutatoreScadenzaFissa(dataScadenza);
        }

        public static IValutatoreScadenza CreatePerScadenzaMensile(int giornoDelMese)
        {
            return new ValutatoreScadenzaMensile(giornoDelMese);
        }

        public static IValutatoreScadenza CreatePerScadenzaSettimanale(DayOfWeek giornoDellaSettimana)
        {
            return new ValutatoreScadenzaSettimanale(giornoDellaSettimana);
        }

        class ValutatoreScadenzaFissa : IValutatoreScadenza
        {
            private readonly DateTime _dataScadenza;

            public ValutatoreScadenzaFissa(DateTime dataScadenza)
            {
                _dataScadenza = dataScadenza.Date;
            }

            public virtual bool AccadeOggi()
            {
                return (DateTime.Today == _dataScadenza);
            }
        }

        class ValutatoreScadenzaMensile : IValutatoreScadenza
        {
            private readonly int _giornoDelMese;

            public ValutatoreScadenzaMensile(int giornoDelMese)
            {
                if (giornoDelMese < 1 || giornoDelMese > 28)
                    throw new ArgumentException("giornoDelMese < 1 || giornoDelMese > 28");
                _giornoDelMese = giornoDelMese;
            }

            public virtual bool AccadeOggi()
            {
                return (DateTime.Today.Day == _giornoDelMese);
            }
        }

        class ValutatoreScadenzaSettimanale : IValutatoreScadenza
        {
            private readonly DayOfWeek _giornoDellaSettimana;

            public ValutatoreScadenzaSettimanale(DayOfWeek giornoDellaSettimana)
            {
                _giornoDellaSettimana = giornoDellaSettimana;
            }

            public virtual bool AccadeOggi()
            {
                return (DateTime.Today.DayOfWeek == _giornoDellaSettimana);
            }
        }
    }
}
