﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

using HomeFinance.Model;

namespace HomeFinance.Persistence
{
    class DocumentoPersister : IDocumentoPersister
    {
        private readonly string _fileName;

        public DocumentoPersister(string fileName)
        {
            _fileName = fileName;
        }

        #region IDocumentoPersister Members

        public IDocumentoLoader GetLoader()
        {
            return new DocumentoLoader(_fileName);
        }

        public void Save(Documento documento)
        {
            DocumentoSaver saver = new DocumentoSaver(_fileName);
            saver.Save(documento);
        }

        #endregion

        private class DocumentoLoader : IDocumentoLoader
        {
            private readonly XmlDocument _xmlDocument;

            public DocumentoLoader(string fileName)
            {
                _xmlDocument = new XmlDocument();
                _xmlDocument.Load(fileName);
            }

            #region IDocumentoLoader Members

            public Categoria LoadCategorie()
            {
                XmlElement rootElement = (XmlElement) _xmlDocument.SelectSingleNode("Documento/Categorie");
                return LoadCategoria(rootElement);
            }

            public List<Movimento> LoadMovimenti()
            {
                return Documento.GetInstance().CreaMovimentiRealiRandom(50).ToList();
            }

            #endregion

            private Categoria LoadCategoria(XmlElement categoriaElement)
            {
                string nome = categoriaElement.GetAttribute("nome");
                Categoria categoria = new Categoria(nome);
                foreach (XmlElement childElement in categoriaElement.SelectNodes("Categoria"))
                {
                    Categoria child = LoadCategoria(childElement);
                    categoria.Add(child);
                }
                return categoria;
            }
        }

        private class DocumentoSaver
        {
            private readonly XmlWriter _writer;

            public DocumentoSaver(string fileName)
            {
                _writer = XmlWriter.Create(fileName, new XmlWriterSettings() { Indent = true });
            }

            public void Save(Documento documento)
            {
                _writer.WriteStartDocument();
                _writer.WriteStartElement("Documento");
                SaveCategoria("Categorie", documento.CategoriaRoot);
                SaveMovimenti(documento.Movimenti);
                _writer.WriteEndElement();  //  Documento
                _writer.WriteEndDocument();
                _writer.Close();
            }

            private void SaveCategoria(string elementName, Categoria categoria)
            {
                _writer.WriteStartElement(elementName);
                _writer.WriteAttributeString("nome", categoria.Nome);
                foreach (Categoria child in categoria.Children)
                {
                    SaveCategoria("Categoria", child);
                }
                _writer.WriteEndElement();  //  elementName
            }

            private void SaveMovimenti(IEnumerable<Movimento> movimenti)
            {
                _writer.WriteStartElement("Movimenti");
                foreach (Movimento movimento in movimenti)
                {
                    SaveMovimento(movimento);
                }
                _writer.WriteEndElement();  //  Movimenti
            }

            private void SaveMovimento(Movimento movimento)
            {
                _writer.WriteStartElement("Movimento");
                //  Salvataggio movimento...
                _writer.WriteEndElement();  //  Movimento
            }
        }
    }
}
