﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using HomeFinance.Model;

namespace HomeFinance.Presentation
{
    public abstract class SelezionePresenter<TControl>
        where TControl : Control
    {
        private readonly TControl _control;
        private readonly ISelezione _selezione;

        public SelezionePresenter(TControl control, ISelezione selezione)
        {
            if (control == null)
                throw new ArgumentNullException("control");
            if (selezione == null)
                throw new ArgumentNullException("selezione");
            _control = control;
            _selezione = selezione;
            _selezione.Changed += OnSelezioneChanged;
            InitializeControl();
            RefreshControl();
        }

        public TControl Control
        {
            get { return _control; }
        }

        public ISelezione Selezione
        {
            get { return _selezione; }
        }

        protected virtual void OnSelezioneChanged(object sender, EventArgs e)
        {
            RefreshControl();
        }

        protected virtual void InitializeControl()
        {
        }

        protected abstract void RefreshControl();
    }
}
