﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using HomeFinance.Model;

namespace HomeFinance.Presentation
{
    public class TreeViewCategoriePresenter
    {
        private readonly TreeView _treeView;

        public event EventHandler CategoriaCorrenteChanged;

        public TreeViewCategoriePresenter(TreeView treeView)
        {
            if (treeView == null)
                throw new ArgumentNullException("treeView");
            _treeView = treeView;
            _treeView.AfterSelect += TreeView_AfterSelect;
            Documento.GetInstance().Changed += Documento_Changed;
            RefreshTreeView();
        }

        public TreeView TreeView
        {
            get { return _treeView; }
        }

        public Categoria CategoriaCorrente
        {
            get
            {
                return (Categoria) ((_treeView.SelectedNode != null) ? _treeView.SelectedNode.Tag : null);
            }
        }

        private void TreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            OnCategoriaCorrenteChanged();
        }

        private void Documento_Changed(object sender, EventArgs e)
        {
            RefreshTreeView();
        }

        private void RefreshTreeView()
        {
            Categoria categoriaRoot = Documento.GetInstance().CategoriaRoot;
            Categoria categoriaCorrente = CategoriaCorrente;
            _treeView.Nodes.Clear();
            AddTreeNodes(_treeView.Nodes, categoriaRoot, categoriaCorrente);
            if (_treeView.SelectedNode == null)
            {
                _treeView.SelectedNode = _treeView.Nodes[0];
            }
        }

        private void AddTreeNodes(TreeNodeCollection treeNodes, Categoria categoria, Categoria categoriaCorrente)
        {
            TreeNode treeNode = new TreeNode(categoria.Nome);
            treeNode.Tag = categoria;
            treeNodes.Add(treeNode);
            foreach (Categoria child in categoria.Children)
            {
                AddTreeNodes(treeNode.Nodes, child, categoriaCorrente);
            }
            if (categoria == categoriaCorrente)
            {
                _treeView.SelectedNode = treeNode;
            }
        }

        protected virtual void OnCategoriaCorrenteChanged()
        {
            if (CategoriaCorrenteChanged != null)
                CategoriaCorrenteChanged(this, EventArgs.Empty);
        }
    }
}
